@@ -143,13 +143,6 @@ class RecordingBase(ABC):
 
     # Compulsory methods to subclass -----------------------------------------
 
-    @abstractmethod
-    def format_measurement(self, data):
-        """How to format the data given by self.Sensor.read().
-
-        Returns a measurement object (e.g. dict, value, custom class etc.)."""
-        pass
-
     @abstractmethod
     def init_file(self, file_manager):
         """How to init the (already opened) data file (columns etc.).
@@ -168,6 +161,12 @@ class RecordingBase(ABC):
 
     # Optional methods to subclass -------------------------------------------
 
+    def format_measurement(self, data):
+        """How to format the data given by self.Sensor.read().
+
+        Returns a measurement object (e.g. dict, value, custom class etc.)."""
+        return data
+
     def after_measurement(self):
         """Define what to do after measurement has been done and formatted.
 
@@ -404,8 +403,13 @@ class RecordBase:
 
     @try_thread
     def cli(self):
-        command_input = ClI(self.recordings, self.properties, self.events)
-        command_input.run()
+        if self.recordings:  # if no recordings provided, no need to record.
+            command_input = ClI(self.recordings, self.properties, self.events)
+            command_input.run()
+        else:
+            self.e_stop.set()
+            raise ValueError('No recordings provided. Stopping ...')
+
 
     @try_thread
     def data_read(self, name):
